/****************************************************************************
 ** rc5.c *******************************************************************
 ****************************************************************************
 *
 * transmit RC5 codes via FTDI bitbang
 * 
 * Copyright (C) 2008 Albert Huitsing <albert@huitsing.nl>
 *
 * Distribute under GPL version 2 or later.
 *
 */

#include <stdio.h>
#include <ftdi.h>

int main(int argc, char **argv)
{
    int ret;
    struct ftdi_context ftdic;
    ftdi_init(&ftdic);
    unsigned char buf[4096];
    int i;
    int bit;
    int msk;
    int car;
    int bufidx;
    unsigned char d;
    int repeat;
    
    int code = 0x3000 | 0;
          

    // receive IR code via command arg:
    if (argc>=2)
    {
      // put 2 startbits in front:
      code = atoi(argv[1]) | 0x3000;
    }
    
    /*
    // just open the first FTDI chip we find:
    if((ret = ftdi_usb_open(&ftdic, 0x0403, 0x6001)) < 0) {
        fprintf(stderr, "unable to open ftdi device: %d (%s)\n", ret, ftdi_get_error_string(&ftdic));
        return EXIT_FAILURE;
    }
    */
    
    // open my FTDI chip (serial number = 'FTE1G5RU' which can be found using 'lsusb')
    if((ret = ftdi_usb_open_desc(&ftdic, 0x0403, 0x6001, NULL, "FTE1G5RU")) < 0) {
        fprintf(stderr, "unable to open ftdi device: %d (%s)\n", ret, ftdi_get_error_string(&ftdic));
        return EXIT_FAILURE;
    }
    
    ret = ftdi_enable_bitbang(&ftdic,0x04);
    if (ret<0)
    {
      fprintf(stderr,"unable to enable bitbang mode\n");
      return EXIT_FAILURE;
    }
    
    // set bitrate for generating 36KHz carrier
    ret = ftdi_set_baudrate(&ftdic,4500);
    if (ret<0)
    {
      fprintf(stderr,"Unable to set required baudrate\n");
      return EXIT_FAILURE;
    }
    
    // prepare 'sample' buffer:
    msk = 0x2000;
    bufidx = 0;
    for (bit=0; bit<14; bit++)
    {
      // each bit has 64 pulses carrier
      for (car=0; car<64; car++)
      {
        d = ((code&msk) ? (car>=32) : (car<32)) ? 255 : 0;
        buf[bufidx++] = d;
        buf[bufidx++] = 0;
      }
      msk>>=1;
    }
    
    for (repeat=0; repeat<10; repeat++)
    {
      ret = ftdi_write_data(&ftdic,buf,bufidx);
      if (ret<0)
      {
        fprintf(stderr,"error buffering block\n");
        return EXIT_FAILURE;
      }
      usleep(100000);
    }

    ftdi_usb_close(&ftdic);
    ftdi_deinit(&ftdic);

    return EXIT_SUCCESS;
}

